<?php
/**
 * File information.
 * Created at: 22:15 - 15/05/2020
 */
namespace KLM;
use Illuminate\Support\Arr;
use Philo\Blade\Blade;

/**
 * Utils.
 * ---------------------------------------------------------------------------------------
 * There are functions will be used by another contexts.
 * ---------------------------------------------------------------------------------------
 *
 * @since 	1.0.0
 */
class Utils {

	/**
	 * @var \Philo\Blade\Blade
	 */
	private static $BLADE;

	/**
	 * Insert new key to an array.
	 * ---------------------------------------------------------------------------------------
	 * Insert new key before or after specific key.
	 * If not found specific key, append it at last position of the array.
	 * If search key is same as insert key, create a new key for search key and insert new
	 * key before that new search key.
	 * ---------------------------------------------------------------------------------------
	 * @param   array                       $array                      The array which need to insert new key.
	 * @param   string|int                  $searchKey                  The specific key need to search.
	 * @param   string|int                  $insertKey                  The new key name.
	 * @param   string|int|array|object     $insertValue                The new key value.
	 * @param   bool                        $insertAfterFoundedKey      Whether to insert new key after that specific key.
	 * @param   bool                        $appendIfNotFound           Whether to append it at last position if not found that specific key.
	 * @param   bool                        $autoPush                   Whether to auto push a new key for exist key if search key is same as insert key.
	 * ---------------------------------------------------------------------------------------
	 * @return  array                       $newArray                   The new array after insert or append new key.
	 * ---------------------------------------------------------------------------------------
	 * @since   1.0.0
	 */
	public static function insertToArray($array, $searchKey, $insertKey, $insertValue, $insertAfterFoundedKey = true, $appendIfNotFound = false, $autoPush = false) {

		// Initialize new array parameter.
		$newArray   = [];
		$countArray = count($array);

		foreach ($array as $key => $value) {

			// Insert before current key?
			// Only if current key is the key we are searching for, and we want to insert before that founded key.
			if ($key === $searchKey && !$insertAfterFoundedKey) {
				$newArray[$insertKey] = $insertValue;
			}

			// If search key is same as insert key and using auto push.
			if ($searchKey === $key && $searchKey == $insertKey && $autoPush) {
				$newArray[$countArray] = $value;
			}

			// Copy the current key/value from old array to a new array.
			else {
				$newArray[$key] = $value;
			}

			// Insert after the current key?
			// Only if current key is the key we are searching for, and we want to insert after that founded key.
			if ($key === $searchKey && $insertAfterFoundedKey){
				$newArray[$insertKey] = $insertValue;
			}

		}

		// Append if key isn't founded.
		if ($appendIfNotFound && count($array) == count($newArray)) {
			$newArray[ $insertKey ] = $insertValue;
		}

		// Return new array.
		return $newArray;

	}

	/**
	 * Include file with parameters.
	 * ---------------------------------------------------------------------------------------
	 * @param   string  $path   The path of file which need to include.
	 * @param   array   $data   The data will be extract and use in included file.
	 * ---------------------------------------------------------------------------------------
	 * @since   1.0.0
	 */
	public static function includeFileWithParameters($path, $data = []) {

		// Replace "." with "/" and append ".php".
		$path = str_replace('.', '/', $path) . '.php';

		// Extract array.
		extract($data);

		// Include file.
		include_once(OPM_PATH . $path);

	}

	/**
	 * Get parameter value from current query.
	 * ---------------------------------------------------------------------------------------
	 * @param   bool    $wp         Whether to use "global $wp" or not.
	 * ---------------------------------------------------------------------------------------
	 * @return  mixed   $params     An array contains all parameters and their value.
	 * ---------------------------------------------------------------------------------------
	 * @since   1.0.0
	 */
	public static function getParamValueCurrentQuery($wp = false) {

		// If we have to use global WordPress, get current query string from URL and WP Request.
		if ($wp) {
			global $wp;
			$currentQuery = add_query_arg([$_GET], $wp->request);
		}

		// Otherwise, only get current query string from URL.
		else {
			$currentQuery = add_query_arg([$_GET], null);
		}

		// If not have any query parameters, stop.
		if (!$currentQuery) return null;

		// Parse query string to array.
		parse_str(parse_url($currentQuery)['query'], $params);

		// Return.
		return $params;

	}

	/**
	 * Number formatter.
	 * ---------------------------------------------------------------------------------------
	 * @param   int|string      $number     The number need to format.
	 * ---------------------------------------------------------------------------------------
	 * @return  int|string      $number     The number has formatted.
	 */
	public static function numberFormat($number) {

		// Return the number formatted.
		return number_format($number, 0, ',', '.');

	}

	/**
	 * Get parameter value from current query.
	 * ---------------------------------------------------------------------------------------
	 * @param		string		$paramName		The paramter name which needed to get value.
	 * @param		bool		$wp				Whether to use "global $wp" or not.
	 * ---------------------------------------------------------------------------------------
	 * @return		string		$value			The value of selected paramater.
	 * ---------------------------------------------------------------------------------------
	 * @since		1.0.0
	 */
	public static function getURLParamValue($paramName = null, $wp = false) {

		// If not have any paramter name, stop.
		if (!$paramName) return null;

		// If we have to use "global $wp", get current query string from URL and WP Request.
		if ($wp) {
			global $wp;
			$currentQuery = add_query_arg([$_GET], $wp->request);

			// Otherwise, only get current query string from URL.
		} else {
			$currentQuery = add_query_arg([$_GET], null);
		}

		// If not have any query parameters, stop.
		if (!$currentQuery) return null;

		// Parse query string to array.
		parse_str(parse_url($currentQuery)['query'], $params);

		// Return parameter value.
		return isset($params[$paramName]) ? $params[$paramName] : null;

	}

	/**
	 * Create a blade view that can be rendered.
	 * ---------------------------------------------------------------------------------------
	 * @param   string $viewName
	 * ---------------------------------------------------------------------------------------
	 * @return  \Illuminate\Contracts\View\View
	 * ---------------------------------------------------------------------------------------
	 * @since   1.0.5
	 */
	public static function view($viewName) {

		// Initialize blade view if it not defined.
		if (!static::$BLADE) {
			$views = __DIR__ . Constants::$relativeViewsDir;
			$cache = __DIR__ . Constants::$relativeCacheDir;
			static::$BLADE = new Blade($views, $cache);
		}

		// Return view.
		return static::$BLADE->view()->make($viewName);

	}

	/**
	 * Get value from an array
	 *
	 * @param array      $array   The array
	 * @param string     $key     Target key
	 * @param null|mixed $default Default value
	 * @return mixed
	 */
	public static function array_get($array, $key, $default = null) {
		return Arr::get($array, $key, $default);
	}

}